#ifndef Split3PM_H
#define Split3PM_H

#define ID3_TAG_SIZE    128
#define MAX_BUFFER 2048

typedef struct
{
//    A 11 (31-21) Frame sync (all bits set) 
    DWORD dwFrameSync;

//    B 2 (20,19) MPEG Audio version
//    00 - MPEG Version 2.5
//    01 - reserved
//    10 - MPEG Version 2
//    11 - MPEG Version 1 
    DWORD dwAudioVersion;

//    C 2 (18,17) Layer description
//    00 - reserved
//    01 - Layer III
//    10 - Layer II
//    11 - Layer I 
    DWORD dwLayerDescription;

//    D 1 (16) Protection bit
//    0 - Protected by CRC (16bit crc follows header)
//    1 - Not protected 
    DWORD dwProtection;

//    E 4 (15,12) Bitrate index
//    bits V1,L1 V1,L2 V1,L3 V2,L1 V2,L2 V2,L3 
//    0000 free free free free free free 
//    0001 32 32 32 32 32 8 (8) 
//    0010 64 48 40 64 48 16 (16) 
//    0011 96 56 48 96 56 24 (24) 
//    0100 128 64 56 128 64 32 (32) 
//    0101 160 80 64 160 80 64 (40) 
//    0110 192 96 80 192 96 80 (48) 
//    0111 224 112 96 224 112 56 (56) 
//    1000 256 128 112 256 128 64 (64) 
//    1001 288 160 128 288 160 128 (80) 
//    1010 320 192 160 320 192 160 (96) 
//    1011 352 224 192 352 224 112 (112) 
//    1100 384 256 224 384 256 128 (128) 
//    1101 416 320 256 416 320 256 (144) 
//    1110 448 384 320 448 384 320 (160) 
//    1111 bad bad bad bad bad bad 
    DWORD dwBitrateIndex;
    DWORD dwBitrate;

//    F 2 (11,10) Sampling rate frequency index (values are in Hz) bits MPEG1 MPEG2 MPEG2.5 
//    00 44100 22050 11025 
//    01 48000 24000 12000 
//    10 32000 16000 8000 
//    11 reserv. reserv. reserv. 
    DWORD dwSampleRateFreq;
    DWORD dwSampleRate;

//    G 1 (9) Padding bit
//    0 - frame is not padded
//    1 - frame is padded with one extra bit 
    DWORD dwPaddingBit;

//    H 1 (8) Private bit (unknown purpose) 
    DWORD dwPrivateBit;

//    I 2 (7,6) Channel Mode
//    00 - Stereo
//    01 - Joint stereo (Stereo)
//    10 - Dual channel (Stereo)
//    11 - Single channel (Mono) 
    DWORD dwChannelMode;

//    J 2 (5,4) Mode extension (Only if Joint stereo)
//    value Intensity stereo MS stereo 
//    00 off off 
//    01 on off 
//    10 off on 
//    11 on on 
    DWORD dwModeExtension;
// 
//    K 1 (3) Copyright
//    0 - Audio is not copyrighted
//    1 - Audio is copyrighted 
    DWORD dwCopyright;

//    L 1 (2) Original
//    0 - Copy of original media
//    1 - Original media 
    DWORD dwOriginal;
   
//    M 2 (1,0) Emphasis
//    00 - none
//    01 - 50/15 ms
//    10 - reserved
//    11 - CCIT J.17 
    DWORD dwEmphasis;

} MP3HEAD, *LPMP3HEAD;

typedef struct 
{
    DWORD x[7];
} BITRATES;


DWORD ExtractBits( DWORD dwSource, DWORD dwFrom, DWORD dwTo );
void FillMp3Head( LPMP3HEAD h, DWORD dwSource );

#define MPEG_Version_2      2
#define MPEG_Version_1      3
#define MPEG_Version_3      0

#define MPEG_Layer_3        1
#define MPEG_Layer_2        2
#define MPEG_Layer_1        3

class MP3File 
    {
    public:
        MP3File();
        virtual ~MP3File();
        BOOL Initialize();
        BOOL LoadFile( LPCSTR lpszFileName );
        void DumpInfo();
        BOOL LocatePosition(DWORD dwSeconds, LPDWORD lpdwPosition);
        LPBYTE FindNextFrameStart(LPBYTE lpbBuffer,LPBYTE lpbMax);
        BOOL CreateTarget(LPCSTR lpszFileName, DWORD dwFrom,DWORD dwTo,BOOL bAppend);

    protected:
        LPBYTE m_lpbCache;
        DWORD m_dwCacheMemorySize;
        BYTE m_bBuffer[MAX_BUFFER];

#ifdef TARGET_IS_UNIX
        int m_hFile;
#else
        HANDLE m_hFile;
#endif
        DWORD m_dwFileSize;
        DWORD m_dwBytesPerFrame;
        DWORD m_dwNumberOfFrames;
        DWORD m_dwBytesPerSecond;
        DWORD m_dwLengthInSeconds;
        MP3HEAD m_Header;
    };

class s3pPart : public PNode
    {
    public:
        s3pPart( DWORD dwFrom, DWORD dwTo, LPCSTR lpszFileName );

        PString m_strFileName;
        DWORD m_dwFrom, m_dwTo;
    };

class s3pFile
    {
    public:
		virtual BOOL Read( LPCSTR lpszFileName );
        virtual BOOL OnParse( LPCSTR lpszLine );

        PList m_Parts;
        PString m_strSource;
    };

#endif // Split3PM_H
